<?php
namespace App\Http\Controllers\Api\V1;

use App\Models\IntroductionInfoModel;
use App\Models\MajorInfoModel;
use App\Models\StudentCountModel;
use App\Models\UserInfoModel;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Services\Base\ErrorCode;
use Validator, Response;
use EasyWeChat\Factory;


class HomeController extends Controller{

    protected $app;

    public function __construct()
    {
        $config = [
            'app_id' => 'wx3cf0f39249eb0exx',
            'secret' => 'f1c242f4f28f735d4687abb469072axx',

            // 指定 API 调用返回结果的类型：array(default)/collection/object/raw/自定义类名
            'response_type' => 'array',
        ];

        $this->app = Factory::miniProgram($config);
    }


    /**
     * @api {post} /api/home/login 登陆(login)
     * @apiDescription 登陆(login)
     * @apiGroup
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {int}    [code](必填)
     * @apiParam {int}    [nickName]
     * @apiParam {int}    [avatar]
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     * {
     *     "status": true,
     *     "status_code": 0,
     *     "message": "",
     *     "data": {
     *         "userinfo": {
     *              "id": "",
     *              "nickname": "",
     *              "openid": "",
     *              "has_agreed": ""           //1:已同意说明，进入主页面；0:为同意协议，进入协议说明页面
     *          }
     *
     *     }
     * }
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     * 可能出现的错误代码：
     *    1000    CLIENT_WRONG_PARAMS             传入参数不正确
     */
    public function login(Request $request){
        $code = $request->get('code');
        $session = $this->app->auth->session($code);

        $openid = $session['openid'];

        $userinfo = UserInfoModel::where('openid',$openid)->first(['id','nickname','openid','has_agreed']);

        if($userinfo){
            return $this->api(compact('userinfo'));
        }else{
            $data['openid'] = $openid;
            $data['nickname'] = $request->get('nickName');
            $data['avatar'] = $request->get('avatar');
            $data['has_agreed'] = 0;

            $userinfo = UserInfoModel::create($data);

            return $this->api(compact('userinfo'));
        }

    }

    /**
     * @api {get} /api/home/getintroduction获取使用说用及协议
     * @apiDescription 获取使用说用及协议
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     * {
     *     "status": true,
     *     "status_code": 0,
     *     "message": "",
     *     "data": {
     *         "list": [
     *
     *          ]
     *
     *     }
     * }
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     * 可能出现的错误代码：
     *    1000    CLIENT_WRONG_PARAMS             传入参数不正确
     */
    public function getIntroduction(){
        $list = IntroductionInfoModel::get(['title','content','type']);

        return $this->api(compact('list'));
    }


    /**
     * @api {post} /api/home/agreeintroduction   同意使用说用及协议
     * @apiDescription 同意使用说用及协议
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {int}    [userid]   用户ID(必填)
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     * {
     *     "status": true,
     *     "status_code": 0,
     *     "message": "",
     *     "data": {
     *         "userinfo": [
     *
     *          ]
     *
     *     }
     * }
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     * 可能出现的错误代码：
     *    1000    CLIENT_WRONG_PARAMS             传入参数不正确
     */
    public function agreeIntroduction(Request $request){
        $userid = $request->get('userid');
        $data['has_agreed'] =1;
        $res = UserInfoModel::where('id',$userid)->update($data);
        if($res){
            $userinfo = UserInfoModel::where('id',$userid)->first(['id','nickname','openid','has_agreed']);
        }

        return $this->api(compact('userinfo'));

    }


    /**
     * @api {get} /api/home/getbasedata   获取可选批次及省份
     * @apiDescription 获取可选批次及省份
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     * {
     *     "status": true,
     *     "status_code": 0,
     *     "message": "",
     *     "data": {
     *         "batchs": [
     *
     *          ],
     *          "provinces":[
     *
     *          ]
     *
     *     }
     * }
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     * 可能出现的错误代码：
     *    1000    CLIENT_WRONG_PARAMS             传入参数不正确
     */
    public function getBaseData(){
        $batchs = MajorInfoModel::groupBy('batch')->pluck('batch');

        $provinces = [
            "全部省份","安徽","澳门","北京","重庆","福建","甘肃","广东","广西","贵州","海南","河北","河南","黑龙江","湖北","湖南","吉林","江苏","江西","辽宁","内蒙古","宁夏","青海","山东","山西","陕西","上海","四川","台湾","天津","西藏","香港","新疆","云南","浙江"
        ];

        return $this->api(compact('batchs','provinces'));
    }


    /**
     * @api {post} /api/home/getqueried   查询免费信息
     * @apiDescription 查询免费信息
     * @apiPermission none
     * @apiVersion 0.1.0
     * @apiParam {int}    [userid]   用户ID(必填)
     * @apiParam {string}    [username]   考生姓名(必填)
     * @apiParam {string}    [cnumber]   考号(必填)
     * @apiParam {int}    [grade]   高考成绩(必填)
     * @apiParam {int}    [batch]   批次(必填)
     * @apiParam {int}    [province]   省份(必填)
     * @apiParam {int}    [code]   推荐码(选填)
     * @apiSuccessExample {json} Success-Response:
     * HTTP/1.1 200 OK
     * {
     *     "status": true,
     *     "status_code": 0,
     *     "message": "",
     *     "data": {
     *         "batchs": [
     *
     *          ],
     *          "provinces":[
     *
     *          ]
     *
     *     }
     * }
     * @apiErrorExample {json} Error-Response:
     * HTTP/1.1 400 Bad Request
     * {
     *     "state": false,
     *     "code": 1000,
     *     "message": "传入参数不正确",
     *     "data": null or []
     * }
     * 可能出现的错误代码：
     *    1000    CLIENT_WRONG_PARAMS             传入参数不正确
     */
    public function getQueried(Request $request){
        $validator = Validator::make($request->all(),
            [
                'cnumber'         => 'required',
                'username'         => 'required',
                'grade'         => 'required|integer',
            ],
            [
                'cnumber.required'      => '考号不能为空!',
                'username.required'    => '姓名不能为空!',
                'username.required'    => '姓名不能为空!',
                'grade.required'    => '成绩不能为空!',
                'grade.integer'    => '请输入正确格式的成绩!',
            ]
        );


        if ($validator->fails()) {
            return $this->error(ErrorCode::CLIENT_WRONG_PARAMS, '传入参数不正确!', $validator->messages());
        }

        /*更新用户的信息*/
        $this->updateUserinfo();

        $year = 2015;
        $grade = $request->get('grade');
        $batch = $request->get('batch');
        $province = $request->get('province');

        $maxgrade = StudentCountModel::where('year',$year)->orderBy('grade','desc')->first()->grade;
        if($grade >$maxgrade){
            $rank = "前10";
        }else{
            $rank = StudentCountModel::where('year',$year)->where('grade',$grade)->first(['total'])->total;
        }

        if($province == "全部省份"){
            $college = MajorInfoModel::where('year',$year)->where('batch',$batch)->where('min_grade',"<=",$grade)->groupBy('college')->get();
        }else{
            $college = MajorInfoModel::where('year',$year)->where('batch',$batch)->where('province','like','%'.$province.'%')->where('min_grade',"<=",$grade)->groupBy('college')->get();
        }

        $college_count = count($college);

        /*创建查询记录*/
        $user = UserInfoModel::find(request('userid'));
        $this-$this->createQueryInfo($user);

        return $this->api(compact('rank','grade','college_count'));
    }




    public function getPaidMajors(){

    }

    /***
     * 更新用户信息
     */
    public function updateUserinfo(){
        $userid = request('userid');

        $data['username'] = request('username');
        $data['mobile'] = request('mobile');
        $data['grade'] = request('grade');
        $data['code'] = request('code');

        $res = UserInfoModel::where('id',$userid)->update($data);

    }

    /***
     * 创建查询记录
     * @param $userinfo
     */
    public function createQueryInfo($userinfo){
        $data['is_paid'] = 0;
        $data['cnumber'] = $userinfo->cnumber;
        $data['grade'] = $userinfo->grade;
        $data['code'] = $userinfo->code;
        $data['mobile'] = $userinfo->mobile;
    }




}